@@ -29,6 +29,10 @@ import (
 
 	"k8s.io/apimachinery/pkg/api/errors"
 
+	"kubevirt.io/kubevirt/tests/framework/checks"
+
+	"kubevirt.io/kubevirt/tests/util"
+
 	storageframework "kubevirt.io/kubevirt/tests/framework/storage"
 
 	expect "github.com/google/goexpect"
@@ -41,18 +45,16 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/rand"
 
-	virtv1 "kubevirt.io/client-go/api/v1"
-	hostdisk "kubevirt.io/kubevirt/pkg/host-disk"
-	. "kubevirt.io/kubevirt/tests/framework/matcher"
-
-	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1alpha1"
-
 	v1 "kubevirt.io/client-go/api/v1"
+	virtv1 "kubevirt.io/client-go/api/v1"
 	"kubevirt.io/client-go/kubecli"
+	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
+	hostdisk "kubevirt.io/kubevirt/pkg/host-disk"
 	virtconfig "kubevirt.io/kubevirt/pkg/virt-config"
 	"kubevirt.io/kubevirt/tests"
 	"kubevirt.io/kubevirt/tests/console"
 	cd "kubevirt.io/kubevirt/tests/containerdisk"
+	. "kubevirt.io/kubevirt/tests/framework/matcher"
 	"kubevirt.io/kubevirt/tests/libnet"
 )
 
@@ -81,13 +83,12 @@ var _ = SIGDescribe("Storage", func() {
 			targetImagePath = tests.HostPathAlpine
 		})
 
-		initNFS := func(targetImage string) *k8sv1.Pod {
-			tests.SkipNFSTestIfRunnigOnKindInfra()
-
+		initNFS := func(targetImage, nodeName string) *k8sv1.Pod {
 			// Prepare a NFS backed PV
 			By("Starting an NFS POD")
 			nfsPod := storageframework.RenderNFSServer("nfsserver", targetImage)
-			nfsPod, err = virtClient.CoreV1().Pods(tests.NamespaceTestDefault).Create(context.Background(), nfsPod, metav1.CreateOptions{})
+			nfsPod.Spec.NodeName = nodeName
+			nfsPod, err = virtClient.CoreV1().Pods(util.NamespaceTestDefault).Create(context.Background(), nfsPod, metav1.CreateOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			Eventually(ThisPod(nfsPod), 180).Should(BeInPhase(k8sv1.PodRunning))
 			nfsPod, err = ThisPod(nfsPod)()
@@ -103,21 +104,23 @@ var _ = SIGDescribe("Storage", func() {
 			nfsIP := libnet.GetPodIpByFamily(nfsPod, ipFamily)
 			ExpectWithOffset(1, nfsIP).NotTo(BeEmpty())
 			os := string(cd.ContainerDiskAlpine)
-			tests.CreateNFSPvAndPvc(pvName, tests.NamespaceTestDefault, "5Gi", nfsIP, os)
+			tests.CreateNFSPvAndPvc(pvName, util.NamespaceTestDefault, "5Gi", nfsIP, os)
 			return pvName
 		}
 
-		runHostPathJobAndExpectCompletion := func(pod *k8sv1.Pod) {
-			pod, err = virtClient.CoreV1().Pods(tests.NamespaceTestDefault).Create(context.Background(), pod, metav1.CreateOptions{})
+		runHostPathJobAndExpectCompletion := func(pod *k8sv1.Pod) *k8sv1.Pod {
+			pod, err = virtClient.CoreV1().Pods(util.NamespaceTestDefault).Create(context.Background(), pod, metav1.CreateOptions{})
+			podWithName := pod.DeepCopy()
 			Expect(err).ToNot(HaveOccurred())
 			Eventually(ThisPod(pod), 120).Should(BeInPhase(k8sv1.PodSucceeded))
 			_, err = ThisPod(pod)()
 			Expect(err).ToNot(HaveOccurred())
+			return podWithName
 		}
 
-		copyAlpineWithNonQEMUPermissions := func() string {
+		copyAlpineWithNonQEMUPermissions := func() (dstPath, nodeName string) {
 
-			dstPath := tests.HostPathAlpine + "-nopriv"
+			dstPath = tests.HostPathAlpine + "-nopriv"
 
 			hostPathType := k8sv1.HostPathDirectoryOrCreate
 
@@ -126,8 +129,9 @@ var _ = SIGDescribe("Storage", func() {
 			By("creating an image with without qemu permissions")
 			pod := tests.RenderHostPathPod("tmp-image-create-job", tests.HostPathBase, hostPathType, k8sv1.MountPropagationNone, []string{"/bin/bash", "-c"}, args)
 
-			runHostPathJobAndExpectCompletion(pod)
-			return dstPath
+			pod = runHostPathJobAndExpectCompletion(pod)
+			nodeName = pod.Spec.NodeName
+			return
 		}
 
 		deleteAlpineWithNonQEMUPermissions := func() {
@@ -142,6 +146,90 @@ var _ = SIGDescribe("Storage", func() {
 
 			runHostPathJobAndExpectCompletion(pod)
 		}
+
+		Context("with error disk", func() {
+			var (
+				nodeName, address, device string
+
+				pvc *k8sv1.PersistentVolumeClaim
+			)
+
+			BeforeEach(func() {
+				nodeName = tests.NodeNameWithHandler()
+				address, device = tests.CreateErrorDisk(nodeName)
+				var err error
+				_, pvc, err = tests.CreatePVandPVCwithFaultyDisk(nodeName, device, util.NamespaceTestDefault)
+				Expect(err).NotTo(HaveOccurred(), "Failed to create PV and PVC for faulty disk")
+			})
+
+			AfterEach(func() {
+				tests.RemoveErrorDisk(nodeName, address)
+			})
+
+			It("[QUARANTINE] should pause VMI on IO error", func() {
+				By("Creating VMI with faulty disk")
+				vmi := tests.NewRandomVMIWithEphemeralDisk(cd.ContainerDiskFor(cd.ContainerDiskAlpine))
+				vmi = tests.AddPVCDisk(vmi, "pvc-disk", "virtio", pvc.Name)
+				_, err := virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Create(vmi)
+				Expect(err).To(BeNil(), "Failed to create vmi")
+
+				tests.WaitForSuccessfulVMIStartWithTimeoutIgnoreWarnings(vmi, 120)
+
+				By("Reading from disk")
+				Expect(console.LoginToAlpine(vmi)).To(Succeed())
+				Expect(console.SafeExpectBatch(vmi, []expect.Batcher{
+					&expect.BSnd{S: "\n"},
+					&expect.BExp{R: console.PromptExpression},
+					&expect.BSnd{S: "nohup sh -c \"sleep 10 && while true; do dd if=/dev/vdb of=/dev/null >/dev/null 2>/dev/null; done\" & \n"},
+					&expect.BExp{R: console.PromptExpression},
+				}, 20)).To(Succeed())
+
+				refresh := ThisVMI(vmi)
+				By("Expecting VMI to be paused")
+				Eventually(func() bool {
+					vmi, err := refresh()
+					Expect(err).ToNot(HaveOccurred())
+
+					for _, condition := range vmi.Status.Conditions {
+						if condition.Type == v1.VirtualMachineInstancePaused {
+							if condition.Status == k8sv1.ConditionTrue {
+								return true
+							}
+							return false
+						}
+					}
+					return false
+
+				}, 100*time.Second, time.Second).Should(BeTrue())
+
+				By("Fixing the device")
+				tests.FixErrorDevice(nodeName)
+
+				By("Expecting VMI to NOT be paused")
+				Eventually(func() bool {
+					vmi, err := refresh()
+					Expect(err).ToNot(HaveOccurred())
+
+					for _, condition := range vmi.Status.Conditions {
+						if condition.Type == v1.VirtualMachineInstancePaused {
+							if condition.Status == k8sv1.ConditionTrue {
+								return true
+							}
+							return false
+						}
+					}
+					return false
+
+				}, 100*time.Second, time.Second).Should(BeFalse())
+
+				By("Cleaning up")
+				err = virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Delete(vmi.ObjectMeta.Name, &metav1.DeleteOptions{})
+				Expect(err).To(BeNil(), "Failed to delete VMI")
+				tests.WaitForVirtualMachineToDisappearWithTimeout(vmi, 120)
+			})
+
+		})
+
 		Context("with faulty disk", func() {
 
 			var (
@@ -155,7 +243,7 @@ var _ = SIGDescribe("Storage", func() {
 				nodeName = tests.NodeNameWithHandler()
 				tests.CreateFaultyDisk(nodeName, deviceName)
 				var err error
-				pv, pvc, err = tests.CreatePVandPVCwithFaultyDisk(nodeName, deviceName, tests.NamespaceTestDefault)
+				pv, pvc, err = tests.CreatePVandPVCwithFaultyDisk(nodeName, "/dev/mapper/"+deviceName, util.NamespaceTestDefault)
 				Expect(err).NotTo(HaveOccurred(), "Failed to create PV and PVC for faulty disk")
 			})
 
@@ -166,10 +254,10 @@ var _ = SIGDescribe("Storage", func() {
 				Expect(err).ToNot(HaveOccurred())
 			})
 
-			It("should pause VMI on IO error", func() {
+			It("[QUARANTINE] should pause VMI on IO error", func() {
 				By("Creating VMI with faulty disk")
 				vmi := tests.NewRandomVMIWithPVC(pvc.Name)
-				_, err := virtClient.VirtualMachineInstance(tests.NamespaceTestDefault).Create(vmi)
+				_, err := virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Create(vmi)
 				Expect(err).To(BeNil(), "Failed to create vmi")
 
 				tests.WaitForSuccessfulVMIStartWithTimeoutIgnoreWarnings(vmi, 120)
@@ -189,7 +277,7 @@ var _ = SIGDescribe("Storage", func() {
 						return false
 					}, 60*time.Second, time.Second).Should(BeTrue())
 
-				err = virtClient.VirtualMachineInstance(tests.NamespaceTestDefault).Delete(vmi.ObjectMeta.Name, &metav1.DeleteOptions{})
+				err = virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Delete(vmi.ObjectMeta.Name, &metav1.DeleteOptions{})
 				Expect(err).To(BeNil(), "Failed to delete VMI")
 				tests.WaitForVirtualMachineToDisappearWithTimeout(vmi, 120)
 			})
@@ -206,19 +294,19 @@ var _ = SIGDescribe("Storage", func() {
 					}
 				})
 				table.DescribeTable("started", func(newVMI VMICreationFunc, storageEngine string, family k8sv1.IPFamily, imageOwnedByQEMU bool) {
-					tests.SkipPVCTestIfRunnigOnKindInfra()
 					if family == k8sv1.IPv6Protocol {
 						libnet.SkipWhenNotDualStackCluster(virtClient)
 					}
 
 					var ignoreWarnings bool
+					var nodeName string
 					// Start the VirtualMachineInstance with the PVC attached
 					if storageEngine == "nfs" {
 						targetImage := targetImagePath
 						if !imageOwnedByQEMU {
-							targetImage = copyAlpineWithNonQEMUPermissions()
+							targetImage, nodeName = copyAlpineWithNonQEMUPermissions()
 						}
-						nfsPod = initNFS(targetImage)
+						nfsPod = initNFS(targetImage, nodeName)
 						pvName = createNFSPvAndPvc(family, nfsPod)
 						ignoreWarnings = true
 					} else {
@@ -234,15 +322,12 @@ var _ = SIGDescribe("Storage", func() {
 					table.Entry("[test_id:3130]with Disk PVC", tests.NewRandomVMIWithPVC, "", nil, true),
 					table.Entry("[test_id:3131]with CDRom PVC", tests.NewRandomVMIWithCDRom, "", nil, true),
 					table.Entry("[test_id:4618]with NFS Disk PVC using ipv4 address of the NFS pod", tests.NewRandomVMIWithPVC, "nfs", k8sv1.IPv4Protocol, true),
-					// Skipping the following 2 tests until https://github.com/kubevirt/kubevirt/issues/4829 is fixed
-					table.PEntry("with NFS Disk PVC using ipv6 address of the NFS pod", tests.NewRandomVMIWithPVC, "nfs", k8sv1.IPv6Protocol, true),
-					table.PEntry("with NFS Disk PVC using ipv4 address of the NFS pod not owned by qemu", tests.NewRandomVMIWithPVC, "nfs", k8sv1.IPv4Protocol, false),
+					table.Entry("[Serial]with NFS Disk PVC using ipv6 address of the NFS pod", tests.NewRandomVMIWithPVC, "nfs", k8sv1.IPv6Protocol, true),
+					table.Entry("[Serial]with NFS Disk PVC using ipv4 address of the NFS pod not owned by qemu", tests.NewRandomVMIWithPVC, "nfs", k8sv1.IPv4Protocol, false),
 				)
 			})
 
 			table.DescribeTable("should be successfully started and stopped multiple times", func(newVMI VMICreationFunc) {
-				tests.SkipPVCTestIfRunnigOnKindInfra()
-
 				vmi = newVMI(tests.DiskAlpineHostPath)
 
 				num := 3
@@ -350,9 +435,8 @@ var _ = SIGDescribe("Storage", func() {
 			var pvc = "empty-pvc1"
 
 			BeforeEach(func() {
-				if !tests.HasFeature(virtconfig.VirtIOFSGate) {
-					Skip("Test requires virtio-fs feature gate")
-				}
+				checks.SkipTestIfNoFeatureGate(virtconfig.VirtIOFSGate)
+				tests.SkipIfNonRoot(virtClient, "VirtioFS")
 				tests.CreateHostPathPv(pvc, filepath.Join(tests.HostPathBase, pvc))
 				tests.CreateHostPathPVC(pvc, "1G")
 			}, 120)
@@ -363,8 +447,6 @@ var _ = SIGDescribe("Storage", func() {
 			}, 120)
 
 			It("should be successfully started and virtiofs could be accessed", func() {
-				tests.SkipPVCTestIfRunnigOnKindInfra()
-
 				pvcName := fmt.Sprintf("disk-%s", pvc)
 				vmi := tests.NewRandomVMIWithPVCFS(pvcName)
 				vmi.Spec.Domain.Resources.Requests[k8sv1.ResourceMemory] = resource.MustParse("512Mi")
@@ -374,13 +456,12 @@ var _ = SIGDescribe("Storage", func() {
 				fs := vmi.Spec.Domain.Devices.Filesystems[0]
 				virtiofsMountPath := fmt.Sprintf("/mnt/virtiof_%s", fs.Name)
 				virtiofsTestFile := fmt.Sprintf("%s/virtiofs_test", virtiofsMountPath)
-				mountVirtiofsCommands := fmt.Sprintf(`
+				mountVirtiofsCommands := fmt.Sprintf(`#!/bin/bash
                                    mkdir %s
                                    mount -t virtiofs %s %s
                                    touch %s
                            `, virtiofsMountPath, fs.Name, virtiofsMountPath, virtiofsTestFile)
-				userData := fmt.Sprintf("%s\n%s", tests.GetFedoraToolsGuestAgentUserData(), mountVirtiofsCommands)
-				tests.AddUserData(vmi, "cloud-init", userData)
+				tests.AddUserData(vmi, "cloud-init", mountVirtiofsCommands)
 
 				vmi = tests.RunVMIAndExpectLaunchIgnoreWarnings(vmi, 300)
 
@@ -391,7 +472,7 @@ var _ = SIGDescribe("Storage", func() {
 				Expect(libnet.WithIPv6(console.LoginToFedora)(vmi)).To(Succeed(), "Should be able to login to the Fedora VM")
 
 				virtioFsFileTestCmd := fmt.Sprintf("test -f /run/kubevirt-private/vmi-disks/%s/virtiofs_test && echo exist", fs.Name)
-				pod := tests.GetRunningPodByVirtualMachineInstance(vmi, tests.NamespaceTestDefault)
+				pod := tests.GetRunningPodByVirtualMachineInstance(vmi, util.NamespaceTestDefault)
 				podVirtioFsFileExist, err := tests.ExecuteCommandOnPod(
 					virtClient,
 					pod,
@@ -405,24 +486,21 @@ var _ = SIGDescribe("Storage", func() {
 		Context("Run a VMI with VirtIO-FS and a datavolume", func() {
 			var dataVolume *cdiv1.DataVolume
 			BeforeEach(func() {
-				if !tests.HasFeature(virtconfig.VirtIOFSGate) {
-					Skip("Test requires virtio-fs feature gate")
-				}
+				checks.SkipTestIfNoFeatureGate(virtconfig.VirtIOFSGate)
+				tests.SkipIfNonRoot(virtClient, "VirtioFS")
 				if !tests.HasCDI() {
 					Skip("Skip DataVolume tests when CDI is not present")
 				}
-				dataVolume = tests.NewRandomDataVolumeWithHttpImport(tests.GetUrl(tests.AlpineHttpUrl), tests.NamespaceTestDefault, k8sv1.ReadWriteOnce)
+				dataVolume = tests.NewRandomDataVolumeWithHttpImport(tests.GetUrl(tests.AlpineHttpUrl), util.NamespaceTestDefault, k8sv1.ReadWriteOnce)
 			})
 			AfterEach(func() {
-				err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(dataVolume.Namespace).Delete(context.Background(), dataVolume.Name, metav1.DeleteOptions{})
+				err = virtClient.CdiClient().CdiV1beta1().DataVolumes(dataVolume.Namespace).Delete(context.Background(), dataVolume.Name, metav1.DeleteOptions{})
 				Expect(err).ToNot(HaveOccurred())
 			})
 
 			It("should be successfully started and virtiofs could be accessed", func() {
-				tests.SkipPVCTestIfRunnigOnKindInfra()
-
 				vmi := tests.NewRandomVMIWithFSFromDataVolume(dataVolume.Name)
-				_, err := virtClient.CdiClient().CdiV1alpha1().DataVolumes(dataVolume.Namespace).Create(context.Background(), dataVolume, metav1.CreateOptions{})
+				_, err := virtClient.CdiClient().CdiV1beta1().DataVolumes(dataVolume.Namespace).Create(context.Background(), dataVolume, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				By("Waiting until the DataVolume is ready")
 				if tests.HasBindingModeWaitForFirstConsumer() {
@@ -436,13 +514,12 @@ var _ = SIGDescribe("Storage", func() {
 				fs := vmi.Spec.Domain.Devices.Filesystems[0]
 				virtiofsMountPath := fmt.Sprintf("/mnt/virtiof_%s", fs.Name)
 				virtiofsTestFile := fmt.Sprintf("%s/virtiofs_test", virtiofsMountPath)
-				mountVirtiofsCommands := fmt.Sprintf(`
+				mountVirtiofsCommands := fmt.Sprintf(`#!/bin/bash
                                        mkdir %s
                                        mount -t virtiofs %s %s
                                        touch %s
                                `, virtiofsMountPath, fs.Name, virtiofsMountPath, virtiofsTestFile)
-				userData := fmt.Sprintf("%s\n%s", tests.GetFedoraToolsGuestAgentUserData(), mountVirtiofsCommands)
-				tests.AddUserData(vmi, "cloud-init", userData)
+				tests.AddUserData(vmi, "cloud-init", mountVirtiofsCommands)
 
 				// with WFFC the run actually starts the import and then runs VM, so the timeout has to include both
 				// import and start
@@ -462,7 +539,7 @@ var _ = SIGDescribe("Storage", func() {
 				}, 30*time.Second)).To(Succeed(), "Should be able to access the mounted virtiofs file")
 
 				virtioFsFileTestCmd := fmt.Sprintf("test -f /run/kubevirt-private/vmi-disks/%s/virtiofs_test && echo exist", fs.Name)
-				pod := tests.GetRunningPodByVirtualMachineInstance(vmi, tests.NamespaceTestDefault)
+				pod := tests.GetRunningPodByVirtualMachineInstance(vmi, util.NamespaceTestDefault)
 				podVirtioFsFileExist, err := tests.ExecuteCommandOnPod(
 					virtClient,
 					pod,
@@ -512,14 +589,13 @@ var _ = SIGDescribe("Storage", func() {
 
 				// The following case is mostly similar to the alpine PVC test above, except using different VirtualMachineInstance.
 				table.DescribeTable("started", func(newVMI VMICreationFunc, storageEngine string, family k8sv1.IPFamily) {
-					tests.SkipPVCTestIfRunnigOnKindInfra()
 					if family == k8sv1.IPv6Protocol {
 						libnet.SkipWhenNotDualStackCluster(virtClient)
 					}
 					var ignoreWarnings bool
 					// Start the VirtualMachineInstance with the PVC attached
 					if storageEngine == "nfs" {
-						nfsPod = initNFS(tests.HostPathAlpine)
+						nfsPod = initNFS(tests.HostPathAlpine, "")
 						pvName = createNFSPvAndPvc(family, nfsPod)
 						ignoreWarnings = true
 					} else {
@@ -539,8 +615,6 @@ var _ = SIGDescribe("Storage", func() {
 
 			// Not a candidate for testing on NFS because the VMI is restarted and NFS PVC can't be re-used
 			It("[test_id:3137]should not persist data", func() {
-				tests.SkipPVCTestIfRunnigOnKindInfra()
-
 				vmi = tests.NewRandomVMIWithEphemeralPVC(tests.DiskAlpineHostPath)
 
 				By("Starting the VirtualMachineInstance")
@@ -605,8 +679,6 @@ var _ = SIGDescribe("Storage", func() {
 
 			// Not a candidate for testing on NFS because the VMI is restarted and NFS PVC can't be re-used
 			It("[test_id:3138]should start vmi multiple times", func() {
-				tests.SkipPVCTestIfRunnigOnKindInfra()
-
 				vmi = tests.NewRandomVMIWithPVC(tests.DiskAlpineHostPath)
 				tests.AddPVCDisk(vmi, "disk1", "virtio", tests.DiskCustomHostPath)
 
@@ -640,7 +712,7 @@ var _ = SIGDescribe("Storage", func() {
 				vmi = tests.NewRandomVMIWithHostDisk("somepath", v1.HostDiskExistsOrCreate, "")
 				virtClient, err := kubecli.GetKubevirtClient()
 				Expect(err).ToNot(HaveOccurred())
-				_, err = virtClient.VirtualMachineInstance(tests.NamespaceTestDefault).Create(vmi)
+				_, err = virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Create(vmi)
 				Expect(err).To(HaveOccurred())
 				Expect(err.Error()).To(ContainSubstring("HostDisk feature gate is not enabled"))
 			})
@@ -649,7 +721,7 @@ var _ = SIGDescribe("Storage", func() {
 				vmi := tests.NewRandomVMIWithFSFromDataVolume("something")
 				virtClient, err := kubecli.GetKubevirtClient()
 				Expect(err).ToNot(HaveOccurred())
-				_, err = virtClient.VirtualMachineInstance(tests.NamespaceTestDefault).Create(vmi)
+				_, err = virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Create(vmi)
 				Expect(err).To(HaveOccurred())
 				Expect(err.Error()).To(ContainSubstring("virtiofs feature gate is not enabled"))
 			})
@@ -657,18 +729,15 @@ var _ = SIGDescribe("Storage", func() {
 
 		Context("[rfe_id:2298][crit:medium][vendor:cnv-qe@redhat.com][level:component] With HostDisk and PVC initialization", func() {
 
-			BeforeEach(func() {
-				if !tests.HasFeature(virtconfig.HostDiskGate) {
-					Skip("Cluster has the HostDisk featuregate disabled, skipping  the tests")
-				}
-			})
-
 			Context("With a HostDisk defined", func() {
 
 				var hostDiskDir string
 				var nodeName string
 
 				BeforeEach(func() {
+					if !checks.HasFeature(virtconfig.HostDiskGate) {
+						Skip("Cluster has the HostDisk featuregate disabled, skipping  the tests")
+					}
 					hostDiskDir = tests.RandTmpDir()
 					nodeName = ""
 				})
@@ -703,7 +772,7 @@ var _ = SIGDescribe("Storage", func() {
 						tests.RunVMIAndExpectLaunch(vmi, 30)
 
 						By("Checking if disk.img has been created")
-						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, tests.NamespaceTestDefault)
+						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, util.NamespaceTestDefault)
 						nodeName = vmiPod.Spec.NodeName
 						output, err := tests.ExecuteCommandOnPod(
 							virtClient,
@@ -726,7 +795,7 @@ var _ = SIGDescribe("Storage", func() {
 						tests.RunVMIAndExpectLaunch(vmi, 30)
 
 						By("Checking if another.img has been created")
-						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, tests.NamespaceTestDefault)
+						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, util.NamespaceTestDefault)
 						nodeName = vmiPod.Spec.NodeName
 						output, err := tests.ExecuteCommandOnPod(
 							virtClient,
@@ -758,10 +827,10 @@ var _ = SIGDescribe("Storage", func() {
 						diskPath = filepath.Join(hostDiskDir, diskName)
 						// create a disk image before test
 						job := tests.CreateHostDiskImage(diskPath)
-						job, err = virtClient.CoreV1().Pods(tests.NamespaceTestDefault).Create(context.Background(), job, metav1.CreateOptions{})
+						job, err = virtClient.CoreV1().Pods(util.NamespaceTestDefault).Create(context.Background(), job, metav1.CreateOptions{})
 						Expect(err).ToNot(HaveOccurred())
 						getStatus := func() k8sv1.PodPhase {
-							pod, err := virtClient.CoreV1().Pods(tests.NamespaceTestDefault).Get(context.Background(), job.Name, metav1.GetOptions{})
+							pod, err := virtClient.CoreV1().Pods(util.NamespaceTestDefault).Get(context.Background(), job.Name, metav1.GetOptions{})
 							Expect(err).ToNot(HaveOccurred())
 							if pod.Spec.NodeName != "" && nodeName == "" {
 								nodeName = pod.Spec.NodeName
@@ -777,7 +846,7 @@ var _ = SIGDescribe("Storage", func() {
 						tests.RunVMIAndExpectLaunch(vmi, 30)
 
 						By("Checking if disk.img exists")
-						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, tests.NamespaceTestDefault)
+						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, util.NamespaceTestDefault)
 						output, err := tests.ExecuteCommandOnPod(
 							virtClient,
 							vmiPod,
@@ -797,7 +866,7 @@ var _ = SIGDescribe("Storage", func() {
 								break
 							}
 						}
-						_, err = virtClient.VirtualMachineInstance(tests.NamespaceTestDefault).Create(vmi)
+						_, err = virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Create(vmi)
 						Expect(err).To(HaveOccurred())
 					})
 				})
@@ -806,7 +875,7 @@ var _ = SIGDescribe("Storage", func() {
 					It("[test_id:852]Should fail to start VMI", func() {
 						By("Starting VirtualMachineInstance")
 						vmi = tests.NewRandomVMIWithHostDisk("/data/unknown.img", "unknown", "")
-						_, err = virtClient.VirtualMachineInstance(tests.NamespaceTestDefault).Create(vmi)
+						_, err = virtClient.VirtualMachineInstance(util.NamespaceTestDefault).Create(vmi)
 						Expect(err).To(HaveOccurred())
 					})
 				})
@@ -814,12 +883,27 @@ var _ = SIGDescribe("Storage", func() {
 
 			Context("With multiple empty PVCs", func() {
 
-				var pvcs = [...]string{"empty-pvc1", "empty-pvc2", "empty-pvc3"}
+				var pvcs = []string{}
+				var node string
+				var nodeSelector map[string]string
 
 				BeforeEach(func() {
+					for i := 0; i < 3; i++ {
+						pvcs = append(pvcs, fmt.Sprintf("empty-pvc-%d-%s", i, rand.String(5)))
+					}
 					for _, pvc := range pvcs {
-						tests.CreateHostPathPv(pvc, filepath.Join(tests.HostPathBase, pvc))
+						hostpath := filepath.Join(tests.HostPathBase, pvc)
+						node = tests.CreateHostPathPv(pvc, hostpath)
 						tests.CreateHostPathPVC(pvc, "1G")
+						if checks.HasFeature(virtconfig.NonRoot) {
+							nodeSelector = map[string]string{"kubernetes.io/hostname": node}
+							By("changing permissions to qemu")
+							args := []string{fmt.Sprintf(`chown 107 %s`, hostpath)}
+							pod := tests.RenderHostPathPod("tmp-change-owner-job", hostpath, k8sv1.HostPathDirectoryOrCreate, k8sv1.MountPropagationNone, []string{"/bin/bash", "-c"}, args)
+
+							pod.Spec.NodeSelector = nodeSelector
+							runHostPathJobAndExpectCompletion(pod)
+						}
 					}
 				}, 120)
 
@@ -831,14 +915,15 @@ var _ = SIGDescribe("Storage", func() {
 				}, 120)
 
 				// Not a candidate for NFS testing because multiple VMIs are started
-				It("[test_id:868]Should initialize an empty PVC by creating a disk.img", func() {
+				It("[test_id:868] Should initialize an empty PVC by creating a disk.img", func() {
 					for _, pvc := range pvcs {
 						By("starting VirtualMachineInstance")
 						vmi = tests.NewRandomVMIWithPVC(fmt.Sprintf("disk-%s", pvc))
+						vmi.Spec.NodeSelector = nodeSelector
 						tests.RunVMIAndExpectLaunch(vmi, 90)
 
 						By("Checking if disk.img exists")
-						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, tests.NamespaceTestDefault)
+						vmiPod := tests.GetRunningPodByVirtualMachineInstance(vmi, util.NamespaceTestDefault)
 						output, err := tests.ExecuteCommandOnPod(
 							virtClient,
 							vmiPod,
@@ -875,12 +960,12 @@ var _ = SIGDescribe("Storage", func() {
 							},
 						},
 					}
-					pod, err = virtClient.CoreV1().Pods(tests.NamespaceTestDefault).Create(context.Background(), pod, metav1.CreateOptions{})
+					pod, err = virtClient.CoreV1().Pods(util.NamespaceTestDefault).Create(context.Background(), pod, metav1.CreateOptions{})
 					Expect(err).NotTo(HaveOccurred())
 
 					By("Waiting for hostPath pod to prepare the mounted directory")
 					Eventually(func() k8sv1.ConditionStatus {
-						p, err := virtClient.CoreV1().Pods(tests.NamespaceTestDefault).Get(context.Background(), pod.Name, metav1.GetOptions{})
+						p, err := virtClient.CoreV1().Pods(util.NamespaceTestDefault).Get(context.Background(), pod.Name, metav1.GetOptions{})
 						Expect(err).ToNot(HaveOccurred())
 						for _, c := range p.Status.Conditions {
 							if c.Type == k8sv1.PodReady {
@@ -901,7 +986,7 @@ var _ = SIGDescribe("Storage", func() {
 
 				configureToleration := func(toleration int) {
 					By("By configuring toleration")
-					cfg := tests.GetCurrentKv(virtClient).Spec.Configuration
+					cfg := util.GetCurrentKv(virtClient).Spec.Configuration
 					cfg.DeveloperConfiguration.LessPVCSpaceToleration = toleration
 					tests.UpdateKubeVirtConfigValueAndWait(cfg)
 				}
@@ -953,7 +1038,6 @@ var _ = SIGDescribe("Storage", func() {
 
 			// Not a candidate for NFS because local volumes are used in test
 			It("[test_id:1015]should be successfully started", func() {
-				tests.SkipPVCTestIfRunnigOnKindInfra()
 				// Start the VirtualMachineInstance with the PVC attached
 				vmi = tests.NewRandomVMIWithPVC(tests.BlockDiskForTest)
 				// Without userdata the hostname isn't set correctly and the login expecter fails...
